#!/usr/bin/env python
import os,sys,wx,datetime, ConfigParser, subprocess

class Com():
    def __init__(self):
        self.plugins = Plugins()                                                      #Plugins can hook to this class and can call other plugins
        self.directory = os.path.dirname(__file__)                                    #The directory where com is located
        self.homedir = os.path.expanduser("~")                                        #The home directory of the user typically formed like "/home/username/"
        self.plugindir = os.path.join(self.directory, "Plugins")                      #The directory where the plugins are located
        self.base_config = Config(os.path.join(self.directory, "configuration.ded"))  #The base configuration file
        self.plugin_config = Config(os.path.join(self.directory, "pluginconf.ded"))   #The plugin configuration file
        self.essentials = ["__init__", "sheetupdate", "dialogs"]                      #List with essential plugins that cannot be disabled
        self.registered = []                                                          #List with initialized modules/plugins
        self.main_module = None                                                       #The main module
        self.main_module_name = None                                                  #The name of the main module
        self.main_class = None                                                        #The main class (which should be part of the main module)
        self.main_class_name = None                                                   #The name of the main class
        sys.path.insert(0, self.plugindir)                                            #Make sure that com can reach the plugins
        self.triggers = []                                                            #List with initialized triggers

    def SetMainModule(self, name):
        """ Set the main instance on which plugins can react """
        self.main_module = self.get_module(name)
        self.main_module_name = name

    def InitMainClass(self, classname, parent = None, id = -1, title = "", pos = wx.DefaultPosition, size = wx.DefaultSize, style = wx.DEFAULT_FRAME_STYLE):
        """ Open the GUI window """
        self.main_class = getattr(self.main_module, classname)(self)
        self.main_class_name = classname
        self.RegisterAlias(self.main_class_name, self.main_class)
        self.main_class.create(parent, id, title, pos, size, style)

    def Init_Plugins(self):
        if self.plugin_config.get("general", "allowed") == "True":
            e = [x.strip(" '") for x in self.plugin_config.get("general", "enabled").split(",")]
            d = [x.strip(" '") for x in self.plugin_config.get("general", "disabled").split(",")]
            for module in os.listdir(self.plugindir):
                mod = module.strip().split(".")
                if "py" in mod and mod[0] != "__init__":
                    mod = mod[0:-1]
                    mod = ".".join(mod)
                    if mod in e:
                        #Register
                        #Add here debug mode switch
                        #try:
                        self.register(mod)
                        self.run("init_dplug", mod, self)
                        #except AttributeError:
                        #    pass
                    elif mod not in d:
                        d.append(module)
                        self.plugin_config.set("general", "disabled", ",".join(d))

    def GetPlugins(self):
        plugs = []
        for module in os.listdir(self.plugindir):
            module = module.strip().split(".")
            if "py" in module and module[0] not in self.essentials:
                module.remove("py")
                module = ".".join(module)
                plugs.append(module)
        return plugs
        
    def Update(self):
        import update
        update.Update(self)

################################_MODULES_################################
    def register(self, name):
        #try:
        mod = __import__(name)
        self.registered.append([name, mod])
        #except:
        #    print str(name) + " failed to register"

    def RegisterAlias(self, name, module):
        """Register a class like a module"""
        self.registered.append([name, module])

    def run(self, func, name = None, args = None):
        """Run a function from a certain module"""
        if name == None: name = self.main_class
        for module in self.registered:
            if module[0] == name:
                if args == None:
                    return getattr(module[1], func)()
                elif type(args) == type(()):
                    return getattr(module[1], func)(*args)
                else:
                    return getattr(module[1], func)(args)

    def get_module(self, name):
        """Return the module object"""
        for mod in self.registered:
            if mod[0] == name:
                return mod[1]

################################_TRIGGERS_################################

    def create(self, trigger):
        self.triggers.append([trigger])

    def add_to(self, trigger, add):
        """trigger [["open_file","project","python"],["title","member","member2"],...]"""
        found = False
        for idx, trig in enumerate(self.triggers):
            if trig[0] == trigger and not found:
                self.triggers[idx].append(add)
                found = True
        if not found:
            self.triggers.append([trigger, add])

    def trigger_update(self, trigger, value = ""):
        """Update the trigger "trigger" if a value is passed this will be passed to the members of the trigger"""
        found = False
        tr = ""
        for trig in self.triggers:
            if trig[0] == trigger and found == False:
                tr = trig
                found = True
        if found:
            for x in tr[1::]:
                if value == "":
                    if self.run("trigger", x, (tr[0],self)) == False:    return False
                else:
                    if self.run("trigger", x, (tr[0],self, value)) == False:    return False

    def trigger_update_all(self):
        for trigger in self.triggers:
            for x in trigger[1::]:
                self.run("trigger", x, self)

################################_COMMANDS_################################

    #Commands can be used to provide you more power by using strong oneliners and also providing macro's
    
    def setup_cmd(self, widget):
        """Setup which widget functions as command line, a wx.TextCtrl is adviced"""
        try:
            self.cmd = widget
        except:
            self.active = False
        else:
            self.active = True
            self.cmdlist = []
    
    def add_cmd(self, cmd, plugin):
        found = False
        for idx, plug in enumerate(self.cmdlist):
            if plug[0] == plugin and not found:
                self.cmdlist[idx].append(plugin)
                found = True
        if not found:
            self.cmdlist.append([cmd, plugin])
            x = self.cmd.GetChoices()
            if x == ["test"]:    x.pop(0)
            x.append(cmd)
            self.cmd.SetChoices(x)
    
    def evaluate(self):
        val = self.cmd.GetValue()
        val = val.strip().split(" ")
        found = False
        for command in self.cmdlist:
            if val[0] == command[0] and found == False:
                for x in command[1::]:
                    self.run("command", x, (val[0], " ".join(val[1::]), self))
                    return True
        if not found:
            return False

################################_KEYBINDINGS_################################
    def register_keybinding(self, project, txt, binding):
        """ register_keybinding("Network", "Connect SSH", "ctrl,shift,s") """
        if binding not in self.get_keybindings():
            self.base_config.set("keybindings", binding, (project, txt))
        else:
            e = [x.strip(" '") for x in self.plugin_config.get("general", "enabled").split(",")]
            if project == "main" and not self.get_project_from_binding(binding) in e:
                self.set_keybinding(project, txt, binding)
            else:
                pass
    
    def is_keybinding(self, key):
        try:
            if self.base_config.get("keybindings", key):    return True
            else:   return False
        except ConfigParser.NoOptionError:
            return False
    
    def get_keybinding(self, project, txt, binding = False):
        for key in self.get_keybindings():
            if eval(str(self.base_config.get("keybindings", key)), {}, {}) == (project, txt):
                return key
        if binding:
            self.register_keybinding(project, txt, binding)
            return self.get_keybinding(project, txt)
        else:
            return ""
    
    def set_keybinding(self, project, txt, binding):
        key = self.get_keybinding(project, txt)
        self.base_config.set("keybindings", binding, (project, txt))
        self.remove_keybinding(key)
    
    def remove_keybinding(self, key):
        self.base_config.remove_option("keybindings", key)
    
    def get_project_from_binding(self, key):
        return eval(str(self.base_config.get("keybindings", key)))[0]
    
    def get_keybindings(self):
        return self.base_config.config.options("keybindings")
    
    def get_project_bindings(self):
        projects = []
        for key in self.get_keybindings():
            plst = eval(str(self.base_config.get("keybindings", key)), {}, {})
            i = False
            for proj in projects:
                if plst[0] in proj:
                    projects[projects.index(proj)].append((plst[1], key))
                    i = True
            if not i:   projects.append([plst[0], (plst[1], key)])
        return projects
    
    def save_bindings(self, keylist):
        """ listformat: [["Network", "Open SSH", "shift+ctrl+p"],[...]] """
        self.base_config.remove_section("keybindings")
        self.base_config.save()
        self.base_config.add_section("keybindings")
        self.base_config.save()
        for x in keylist: self.register_keybinding(x[0], x[1], x[2])
        #3 Saves in a row because of section removal

class Config():
    def __init__(self, location):
        self.location = location
        self.config = ConfigParser.ConfigParser()
        self.config.read(self.location)

    def save(self):
        with open(self.location,"wb") as configf:
            self.config.write(configf)

    def set(self, section, name, value):
        self.config.set(section, name, value)
        self.save()

    def get(self, section, name):
        if self.config.has_option(section, name):
            return self.config.get(section, name)
        else:
            return False
    
    def add_section(self, section):
        if not self.config.has_section(section):
            self.config.add_section(section)
            self.save()
        else:
            return False
    
    def remove_section(self, section):
        if self.config.has_section(section):
            self.config.remove_section(section)
            self.save()
        else:
            return False
    
    def remove_option(self, section, option):
        if self.config.has_section(section) and self.config.has_option(section, option):
            self.config.remove_option(section, option)
        else:
            return False

################################_PLUGINS_################################
class Plugins():
    """this is just an empty class on which plugins can bind their values"""
    def __init__(self):
        pass
